/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.objectweb.petals.tools.jmx.api;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.DeploymentServiceMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;

/**
 * Client API of the JMX deployment service.
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsDeploymentServiceClient
		extends
			PetalsAbstractServiceClient {

	/**
	 * Action : deploy
	 */
	protected static final String DEPLOY = "deploy";

	/**
	 * Action : start
	 */
	protected static final String START = "start";

	/**
	 * Action : getDeployedServiceAssemblies
	 */
	protected static final String GET_DEPLOYED_SA = "getDeployedServiceAssemblies";

	/**
	 * Action : getDeployedServiceAssembliesForComponent
	 */
	protected static final String GET_DEPLOYED_SA_FOR_COMPONENT = "getDeployedServiceAssembliesForComponent";

	private final static String DEPLOYMENT_SERVICE_JMX_NAME = "Deployment";

	/**
	 * Creates a new instance of {@link PetalsDeploymentServiceClient}
	 * 
	 */
	public PetalsDeploymentServiceClient(String petalsDomain,
			MBeanServerConnection mBeanServerConnection)
			throws DeploymentServiceDoesNotExistException,
			DeploymentServiceErrorException {

		super(petalsDomain, mBeanServerConnection);

		try {
			Set objNames;

			Hashtable<String, String> attributes = new Hashtable<String, String>();
			attributes.put(JMX_FIELD_NAME, DEPLOYMENT_SERVICE_JMX_NAME);
			attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
			ObjectName objName = new ObjectName(petalsDomain, attributes);
			objNames = this.mBeanServerConnection.queryNames(objName, null);
			if (objNames != null && objNames.size() == 1) {
				this.mbeanName = (ObjectName) objNames.iterator().next();
			} else {
				throw new DeploymentServiceDoesNotExistException();
			}
		} catch (MalformedObjectNameException e) {
			throw new DeploymentServiceErrorException(e);
		} catch (IOException e) {
			throw new DeploymentServiceErrorException(e);
		}
	}

	/**
	 * Check if a service assembly is deployed.
	 * 
	 * @param serviceAssembly
	 *            The service assembly identifier
	 * @throws PerformActionErrorException
	 * @return true if the service assembly; false if the service assembly is
	 *         not deploy or if <code>serviceAssembly</code> is null.
	 */
	public boolean isServiceAssemblyDeployed(String serviceAssembly)
			throws PerformActionErrorException {

		boolean bRet = false;

		String[] deployedSA = this.getDeployedServiceAssemblies();
		for (int i = 0; i < deployedSA.length; i++) {
			if (deployedSA[i].equals(serviceAssembly)) {
				bRet = true;
				break;
			}
		}

		return bRet;
	}

	/**
	 * Get deployed service assemblies.
	 * 
	 * @throws PerformActionErrorException
	 */
	public String[] getDeployedServiceAssemblies()
			throws PerformActionErrorException {

		return (String[]) this.performAction(GET_DEPLOYED_SA, null, null);
	}

	/**
	 * Get service assemblies containing service units running on a component.
	 * 
	 * @param componentName
	 * @throws PerformActionErrorException
	 */
	public String[] getDeployedServiceAssembliesForComponent(
			String componentName) throws PerformActionErrorException {

		Object[] objectsLoad = new Object[1];
		objectsLoad[0] = componentName;
		String[] stringsLoad = new String[1];
		stringsLoad[0] = "java.lang.String";

		return (String[]) this.performAction(GET_DEPLOYED_SA_FOR_COMPONENT,
				objectsLoad, stringsLoad);
	}

	/**
	 * Deploy a service assembly.
	 * 
	 * @see DeploymentServiceMBean#deploy(String)
	 * @param packageURL
	 * @throws PerformActionErrorException
	 */
	public String deploy(URL packageURL) throws PerformActionErrorException {

		Object[] objectsLoad = new Object[1];
		objectsLoad[0] = packageURL.toString();
		String[] stringsLoad = new String[1];
		stringsLoad[0] = "java.lang.String";

		return (String) this.performAction(DEPLOY, objectsLoad, stringsLoad);
	}

	/**
	 * Start a service assembly.
	 * 
	 * @see DeploymentServiceMBean#deploy(String)
	 * @param packageURL
	 * @throws PerformActionErrorException
	 */
	public String start(String serviceAssemblyName)
			throws PerformActionErrorException {

		Object[] objectsLoad = new Object[1];
		objectsLoad[0] = serviceAssemblyName;
		String[] stringsLoad = new String[1];
		stringsLoad[0] = "java.lang.String";

		return (String) this.performAction(START, objectsLoad, stringsLoad);
	}
}
