/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
package org.objectweb.petals.tools.jmx.api;

import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.InstallationServiceMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.petals.tools.jmx.api.exception.InstallationServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.InstallationServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;

/**
 * Client API of the JMX installation service.
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsInstallationServiceClient
		extends
			PetalsAbstractServiceClient {

	/**
	 * Action : installSharedLibrary
	 */
	protected static final String INSTALL_SHARED_LIBRARY = "installSharedLibrary";

	/**
	 * Action : loadNewInstaller
	 */
	protected static final String LOAD_NEW_INSTALLED = "loadNewInstaller";

	/**
	 * Action : loadNewInstaller
	 */
	protected static final String UNLOAD_INSTALLER = "unloadInstaller";

	/**
	 * Action : install
	 */
	protected static final String INSTALL = "install";

	/**
	 * Action : uninstall
	 */
	protected static final String UNINSTALL = "uninstall";

	private final static String INSTALLATION_SERVICE_JMX_NAME = "Installation";

	/**
	 * Creates a new instance of {@link PetalsInstallationServiceClient}
	 * 
	 */
	public PetalsInstallationServiceClient(String petalsDomain,
			MBeanServerConnection mBeanServerConnection)
			throws InstallationServiceDoesNotExistException,
			InstallationServiceErrorException {

		super(petalsDomain, mBeanServerConnection);

		try {
			Set objNames;

			Hashtable<String, String> attributes = new Hashtable<String, String>();
			attributes.put(JMX_FIELD_NAME, INSTALLATION_SERVICE_JMX_NAME);
			attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
			ObjectName objName = new ObjectName(petalsDomain, attributes);
			objNames = this.mBeanServerConnection.queryNames(objName, null);
			if (objNames != null && objNames.size() == 1) {
				this.mbeanName = (ObjectName) objNames.iterator().next();
			} else {
				throw new InstallationServiceDoesNotExistException();
			}
		} catch (MalformedObjectNameException e) {
			throw new InstallationServiceErrorException(e);
		} catch (IOException e) {
			throw new InstallationServiceErrorException(e);
		}
	}

	/**
	 * Load a new installer.
	 * 
	 * @see InstallationServiceMBean#loadNewInstaller(String)
	 * @param packageURL
	 * @throws PerformActionErrorException
	 */
	public PetalsInstallerComponentClient loadNewInstaller(URL packageURL)
			throws PerformActionErrorException {

		Object[] objectsLoad = new Object[1];
		objectsLoad[0] = packageURL.toString();
		String[] stringsLoad = new String[1];
		stringsLoad[0] = "java.lang.String";

		Object obj = this.performAction(LOAD_NEW_INSTALLED, objectsLoad,
				stringsLoad);
		if (!(obj instanceof ObjectName)) {
			throw new PerformActionErrorException("Unexpected returned object");
		}
		return new PetalsInstallerComponentClient(this.petalsDomain,
				(ObjectName) obj, this.mBeanServerConnection);
	}

	/**
	 * Unload a component installer. The component is to be deleted as well.
	 * 
	 * @see InstallationServiceMBean#unloadInstaller(String, boolean)
	 * @throws PerformActionErrorException
	 */
	public void unloadInstaller(String componentName)
			throws PerformActionErrorException {

		Object[] objectsLoad = new Object[2];
		objectsLoad[0] = componentName;
		objectsLoad[1] = true;
		String[] stringsLoad = new String[2];
		stringsLoad[0] = "java.lang.String";
		stringsLoad[1] = "boolean";

		this.performAction(UNLOAD_INSTALLER, objectsLoad, stringsLoad);
	}

}
