/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package org.objectweb.petals.tools.jmx.api;

import java.io.IOException;
import java.util.Set;

import javax.jbi.management.InstallerMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.petals.tools.jmx.api.exception.AttributeErrorException;
import org.objectweb.petals.tools.jmx.api.exception.ComponentErrorException;
import org.objectweb.petals.tools.jmx.api.exception.InstallerComponentDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;

/**
 * Client API of a JMX component associated to a JBI component installer.
 * 
 * @author cdeneux - Capgemini Sud
 */
public final class PetalsInstallerComponentClient
		extends
			PetalsAbstractServiceClient {

	/**
	 * Action : init
	 */
	protected static final String INIT = "init";

	/**
	 * Action : install
	 */
	protected static final String INSTALL = "install";

	/**
	 * Action : uninstall
	 */
	protected static final String UNINSTALL = "uninstall";

	/**
	 * Attribut : installed
	 */
	protected static final String INSTALLED = "Installed";

	/**
	 * Creates a new instance of {@link PetalsInstallerComponentClient} finding
	 * the MBean associated to the component name
	 * 
	 */
	public PetalsInstallerComponentClient(String petalsDomain,
			String componentName, MBeanServerConnection mBeanServerConnection)
			throws InstallerComponentDoesNotExistException,
			ComponentErrorException {

		super(petalsDomain, mBeanServerConnection);

		try {
			Set objNames;

			String pattern = "*:type=installer,name=" + componentName;
			ObjectName objName = new ObjectName(pattern);
			objNames = this.mBeanServerConnection.queryNames(objName, null);
			if (objNames != null && objNames.size() == 1) {
				this.mbeanName = (ObjectName) objNames.iterator().next();
			} else {
				throw new InstallerComponentDoesNotExistException(componentName);
			}

		} catch (MalformedObjectNameException e) {
			throw new ComponentErrorException(e);
		} catch (IOException e) {
			throw new ComponentErrorException(e);
		}
	}

	/**
	 * Creates a new instance of {@link PetalsInstallerComponentClient} from its
	 * MBean
	 * 
	 */
	public PetalsInstallerComponentClient(String petalsDomain,
			ObjectName objectName, MBeanServerConnection mBeanServerConnection) {

		super(petalsDomain, mBeanServerConnection);
		this.mbeanName = objectName;

	}

	/**
	 * Install the component.
	 * 
	 * @see InstallerMBean#install()
	 * @throws PerformActionErrorException
	 */
	public void install() throws PerformActionErrorException {

		Object[] objects = new Object[0];
		String[] strings = new String[0];
		this.performAction(INSTALL, objects, strings);
	}

	/**
	 * Uninstall the component.
	 * 
	 * @see InstallerMBean#uninstall()
	 * @throws PerformActionErrorException
	 */
	public void uninstall() throws PerformActionErrorException {

		Object[] objects = new Object[0];
		String[] strings = new String[0];
		this.performAction(UNINSTALL, objects, strings);
	}

	/**
	 * Get the current state of the JBI component associated to this JMX
	 * component.
	 * 
	 * @return the current state of the JBI component.
	 * @throws AttributeErrorException
	 */
	public boolean isInstalled() throws AttributeErrorException {

		return (Boolean) this.getAttribute(INSTALLED);

	}
}
